-- BuffCheck_Core.lua
-- by Mithrandir1210

--[[ TO-DO LIST:
	
	- 
	 
]]

-- GENERAL INFORMATION --======================================================

-- Load addon-local variables table
local BuffCheck_Name, bc = ...
	-- BuffCheck_Name = "BuffCheck"
	-- BuffCheck = {}

	
bc.Description = {
	Title = GetAddOnMetadata(BuffCheck_Name, "Title"),
	VersionNum = GetAddOnMetadata(BuffCheck_Name, "Version"),
	Author = GetAddOnMetadata(BuffCheck_Name, "Author"),
	LastUpdated = "12/8/2015"
}

-- INITIALIZATION --===========================================================

-- CREATE FRAMES --
bc.EventFrame = CreateFrame("Frame")

-- REGISTER EVENTS --
bc.EventFrame:RegisterEvent("ADDON_LOADED")
bc.EventFrame:RegisterEvent("READY_CHECK_FINISHED")

-- POPULATE INTERFACE OPTIONS --===============================================

-- Add entry to interface options
bc.panel = CreateFrame ("Frame", "bc_Panel", UIParent);
bc.panel.name = "Buff Check";
InterfaceOptions_AddCategory(bc.panel);

-- Create title (header)
bc.SettingsTitle = bc.panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
bc.SettingsTitle:SetPoint("TOPLEFT", 10, -15)
bc.SettingsTitle:SetText("Buff Check Options")

-- Create checkbuttons and their labels

-- 1. Disable output?
bc.btnDisableOutput = CreateFrame ("CheckButton", nil, bc.panel, "InterfaceOptionsCheckButtonTemplate")
bc.btnDisableOutput:SetPoint("TOPLEFT", 20, -40)
bc.btnDisableOutput_Text = bc.btnDisableOutput:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
bc.btnDisableOutput_Text:SetPoint("LEFT", bc.btnDisableOutput, "RIGHT", 0, -1)
bc.btnDisableOutput_Text:SetText("Disable all output")

-- 2. Announce to raid?
bc.btnAnnounce = CreateFrame ("CheckButton", nil, bc.panel, "InterfaceOptionsCheckButtonTemplate")
bc.btnAnnounce:SetPoint("TOPLEFT", 20, -70)
bc.btnAnnounce_Text = bc.btnAnnounce:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
bc.btnAnnounce_Text:SetPoint("LEFT", bc.btnAnnounce, "RIGHT", 0, -1)
bc.btnAnnounce_Text:SetText("Announce missing buffs to chat")

-- 3. Check after readycheck?
bc.btnReadycheck = CreateFrame ("CheckButton", nil, bc.panel, "InterfaceOptionsCheckButtonTemplate")
bc.btnReadycheck:SetPoint("TOPLEFT", 20, -100)
bc.btnReadycheck_Text = bc.btnReadycheck:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
bc.btnReadycheck_Text:SetPoint("LEFT", bc.btnReadycheck, "RIGHT", 0, -1)
bc.btnReadycheck_Text:SetText("Check for buffs after readycheck")

-- 4. Display settings on load?
bc.btnDisplaySettingsOnLoad = CreateFrame ("CheckButton", nil, bc.panel, "InterfaceOptionsCheckButtonTemplate")
bc.btnDisplaySettingsOnLoad:SetPoint("TOPLEFT", 20, -130)
bc.btnDisplaySettingsOnLoad_Text = bc.btnDisplaySettingsOnLoad:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
bc.btnDisplaySettingsOnLoad_Text:SetPoint("LEFT", bc.btnDisplaySettingsOnLoad, "RIGHT", 0, -1)
bc.btnDisplaySettingsOnLoad_Text:SetText("Display settings on load")

-- Announce as raid warning (only works if "announce to raid" is enabled)
-- <to be implemented!!!>

-- FUNCTIONS --================================================================

-------------------------------------------------------------------------------
-- Displays a message to chat or locally to the console if no ChatType or
--   ChatType == "console" (void).
-- Param1: Message to display (string);
-- Param2 (optional): Send message in chat or console ("RAID", "SAY", "console", etc)
-- Param3 (optional): Color of text to display (only available for console)
-------------------------------------------------------------------------------
bc.Display = function(MessageString, ChatType, FontColor)
	
	-- Set color to default if no color is specified
	if not FontColor then
		FontColor = bc.Defaults.FontColor
	end
	
	-- If a chat type is specified, use SendChatMessage() instead of print()
	-- Display to chat
	if ChatType and ChatType ~= "console" then
	    MessageString = "[BC] " .. MessageString
        SendChatMessage(MessageString, ChatType, nil, nil)
	-- Display to console
    elseif not ChatType or ChatType == "console" then
	    MessageString = FontColor.."[BC] "..MessageString
	    print(MessageString)
    else
		MessageString = FontColor.."[BC] "..MessageString
	    print(MessageString)
	end
    
end

-------------------------------------------------------------------------------
-- Uses the 'bc.Buffs' table to display missing buffs to the proper channel (void).
-------------------------------------------------------------------------------
bc.PrintMissingBuffs = function (Channel)
	
	-- Set and error-check channel type
	if Channel == "RAID" or Channel == "raid" or Channel == "Raid" then
		Channel = "RAID"
	elseif Channel == "PARTY" or Channel == "party" or Channel == "Party" then
		Channel = "PARTY"
	elseif Channel == "SOLO" or Channel == "solo" or Channel == "Solo" then
		Channel = "SOLO"
	else
		Channel = "console"
	end
	
	-- Display to "SAY" or console based on settings
	if not BuffCheck_ActiveSettings.AnnounceMissing then
		Channel = "console"
	elseif BuffCheck_ActiveSettings.AnnounceMissing and Channel == "SOLO" then
		Channel = "SAY"
	end
	
	if not BuffCheck_ActiveSettings.DisableOutput then
	
		-- Print list of players missing buffs (if any buffs are missing)
		local BuffType = nil
		
		if not bc.Buffs.AllBuffsFound() then
			bc.Display("Checking for missing buffs...",Channel)
			for i=1, #bc.Buffs.List do
				BuffType = tostring(bc.Buffs.List[i])
				bc.Buffs[BuffType]["DisplayMissing"](Channel)
			end
		elseif bc.Buffs.AllBuffsFound() then
			bc.Display("All buffs found.",Channel)
		else
			-- Display directly to console
			bc.Display("Something went wrong.", "console")
		end
	end

end

-------------------------------------------------------------------------------
-- Uses the 'bc.Buffs' table to check for buffs based on zone (void).
-- Calls PrintMissingBuffs().
-------------------------------------------------------------------------------
bc.CheckBuffs = function()
	
	-- Raid group
    if IsInRaid() then
        bc.NumPlayers = GetNumGroupMembers()
        
		local BuffType = nil
		
        for i=1, bc.NumPlayers do
			for j=1, #bc.Buffs.List do
				BuffType = bc.Buffs.List[j]
				bc.Buffs[BuffType]["Check"]("raid"..i)
			end
        end
    
        bc.PrintMissingBuffs("RAID")
    
	-- Party
	elseif not IsInRaid() and IsInGroup(LE_PARTY_CATEGORY_HOME) then
		bc.NumPlayers = GetNumGroupMembers()
        
		local BuffType = nil
		
        for i=1, bc.NumPlayers do
            for j=1, #bc.Buffs.List do
				BuffType = bc.Buffs.List[j]
				bc.Buffs[BuffType]["Check"]("party"..i)
			end
        end
		
		for j=1, #bc.Buffs.List do
			BuffType = bc.Buffs.List[j]
			bc.Buffs[BuffType]["Check"]("player")
		end
    
        bc.PrintMissingBuffs("PARTY")
    
	-- Instance group
	elseif IsInGroup(LE_PARTY_CATEGORY_INSTANCE) then
		bc.NumPlayers = GetNumGroupMembers()
        
		local BuffType = nil
		
        for i=1, bc.NumPlayers do
            
            for j=1, #bc.Buffs.List do
				BuffType = bc.Buffs.List[j]
				bc.Buffs[BuffType]["Check"]("raid"..i)
			end
        end
    
        bc.PrintMissingBuffs("RAID")
	
	-- Solo or other
    else
        
		local BuffType = nil
		
		for j=1, #bc.Buffs.List do
			BuffType = bc.Buffs.List[j]
			bc.Buffs[BuffType]["Check"]("player")
		end
		
        bc.PrintMissingBuffs("SOLO")
		
    end -- if IsInRaid
    
end -- CheckBuffs()

-------------------------------------------------------------------------------
-- Print current settings to the console (void).
-------------------------------------------------------------------------------
bc.ViewSettings = function()

	-- 1. Disable output
	if BuffCheck_ActiveSettings.DisableOutput then
		bc.Display("Disable output?: ".."Yes")
	else
		bc.Display("Disable output?: ".."No")
	end
	
	-- 2. Announce missing 
	if BuffCheck_ActiveSettings.AnnounceMissing then
		bc.Display("Announce to chat?: ".."Yes")
	else
		bc.Display("Announce to chat?: ".."No")
	end
	
	-- 3. Check after readycheck
	if BuffCheck_ActiveSettings.CheckAfterReadycheck then
		bc.Display("Check buffs at readycheck?: ".."Yes")
	else
		bc.Display("Check buffs at readycheck?: ".."No")
	end
	
	-- 4. Display settings on load
	if BuffCheck_ActiveSettings.DisplaySettingsOnLoad then
		bc.Display("Display settings on load?: ".."Yes")
	else
		bc.Display("Display settings on load?: ".."No")
	end
	
end

-------------------------------------------------------------------------------
-- Reset settings to their defaults (void).
-------------------------------------------------------------------------------
bc.SetDefaults = function()
	
	bc.Buffs.Clear()
		
	-- Disable output
	BuffCheck_ActiveSettings.DisableOutput = bc.Defaults.DisableOutput
	bc.btnDisableOutput:SetChecked(bc.Defaults.DisableOutput)	
		
	-- Announce to group
	BuffCheck_ActiveSettings.AnnounceMissing = bc.Defaults.AnnounceMissing
	bc.btnAnnounce:SetChecked(bc.Defaults.AnnounceMissing)
	
	-- Check after readycheck
	BuffCheck_ActiveSettings.CheckAfterReadycheck = bc.Defaults.CheckAfterReadycheck
	bc.btnReadycheck:SetChecked(bc.Defaults.CheckAfterReadycheck)

	-- Display settings on load
	BuffCheck_ActiveSettings.DisplaySettingsOnLoad = bc.Defaults.DisplaySettingsOnLoad
	bc.btnDisplaySettingsOnLoad:SetChecked(bc.Defaults.DisplaySettingsOnLoad)
	
	bc.Display("Defaults were set to the following values: ")
	bc.ViewSettings()
	
end

-- SLASH COMMANDS --===========================================================

bc.SlashCmdHandler = function(msg)
	if msg == "check" then
		bc.Buffs.Clear()  -- moved to bc.BuffCheck(); untested
	    bc.CheckBuffs()
	elseif msg == "announce" then
		BuffCheck_ActiveSettings.AnnounceMissing = true
		bc.btnAnnounce:SetChecked(true) -- Set checkbutton state
		bc.Display("Missing buffs will be displayed in CHAT.")
	elseif msg == "noannounce" then
		BuffCheck_ActiveSettings.AnnounceMissing = false
		bc.btnAnnounce:SetChecked(false) -- Set checkbutton state
		bc.Display("Missing buffs will be displayed LOCALLY.")
	elseif msg == "readycheck" then
		BuffCheck_ActiveSettings.CheckAfterReadycheck = true
		bc.btnReadycheck:SetChecked(true) -- Set checkbutton state
		bc.Display("Buffs WILL be checked after a readycheck occurs.")
	elseif msg == "noreadycheck" then
		BuffCheck_ActiveSettings.CheckAfterReadycheck = false
		bc.btnReadycheck:SetChecked(false) -- Set checkbutton state
		bc.Display("Buffs will NOT be checked after a readycheck occurs.")
	elseif msg == "enableoutput" then
		BuffCheck_ActiveSettings.DisableOutput = false
		bc.btnDisableOutput:SetChecked(false)
		bc.Display("Missing buffs WILL be displayed.")
	elseif msg == "disableoutput" then
		BuffCheck_ActiveSettings.DisableOutput = true
		bc.btnDisableOutput:SetChecked(true)
		bc.Display("Missing buffs will NOT be displayed.")
	elseif msg == "settingsonload" then
		BuffCheck_ActiveSettings.DisplaySettingsOnLoad = true
		bc.btnDisplaySettingsOnLoad:SetChecked(true)
		bc.Display("Settings WILL be displayed on load.")
	elseif msg == "nosettingsonload" then
		BuffCheck_ActiveSettings.DisplaySettingsOnLoad = false
		bc.btnDisplaySettingsOnLoad:SetChecked(false)
		bc.Display("Settings will NOT be displayed on load.")
	elseif msg == "reset" then
		bc.SetDefaults()
	elseif msg == "viewsettings" then
		bc.ViewSettings()
	elseif msg == "settings" or msg == "options" then
		InterfaceOptionsFrame_OpenToCategory(bc.panel.name)
		InterfaceOptionsFrame_OpenToCategory(bc.panel.name)
	elseif msg == "help" then
	    bc.Display("'/bc check' - Check buffs.")
		bc.Display("'/bc announce' - Display missing buffs in CHAT.")
		bc.Display("'/bc noannounce' - Display missing buffs LOCALLY.")
		bc.Display("'/bc readycheck' - ENABLE display of missing buffs after a readycheck.")
		bc.Display("'/bc noreadycheck' - DISABLE display of missing buffs after a readycheck.")
		bc.Display("'/bc enableoutput' - ENABLE display of missing buffs.")
		bc.Display("'/bc disableoutput' - DISABLE display of missing buffs.")
		bc.Display("'/bc settingsonload' - ENABLE display of settings on load.")
		bc.Display("'/bc nosettingsonload' - DISABLE display of settings on load.")
		bc.Display("'/bc viewsettings' - View current settings.")
		bc.Display("'/bc settings' or '/bc options' - Open interface options.")
		bc.Display("'/bc reset' - RESET settings to defaults.")
	elseif msg == "test" then
	    -- Debug and test functions here --
    else
		bc.Display("Type '/bc help' for a command listing.")
	end
		
end -- SlashCmdHandler()

SLASH_BUFFCHECK1 = "/bc"
SLASH_BUFFCHECK2 = "/buffcheck"
SlashCmdList["BUFFCHECK"] = bc.SlashCmdHandler

-- EVENT HANDLERS --===========================================================

-------------------------------------------------------------------------------
-- Sets global defaults if none are specified in the settings.
-- Calls CheckBuffs() on readycheck event if settings call for it.
-------------------------------------------------------------------------------
bc.EventFrame:SetScript("OnEvent", function(self, event, addon, ...)
	
	if event == "ADDON_LOADED" and addon == "BuffCheck" then
		
		-- Disable output?
		if BuffCheck_ActiveSettings.DisableOutput == nil then
			-- Set active settings to defaults
			BuffCheck_ActiveSettings.DisableOutput = bc.Defaults.DisableOutput
			bc.btnReadycheck:SetChecked(bc.Defaults.DisableOutput)
		end
		-- Set state of checkbutton within interface options
		bc.btnDisableOutput:SetChecked(BuffCheck_ActiveSettings.DisableOutput)
		
		-- Announce to chat?
		if BuffCheck_ActiveSettings.AnnounceMissing == nil then
			-- Set saved settings to defaults
			BuffCheck_ActiveSettings.AnnounceMissing = bc.Defaults.AnnounceMissing
			bc.btnAnnounce:SetChecked(bc.Defaults.AnnounceMissing)
		end
		-- Set state of checkbutton within interface options
		bc.btnAnnounce:SetChecked(BuffCheck_ActiveSettings.AnnounceMissing)
		
		-- Check after readycheck?
		if BuffCheck_ActiveSettings.CheckAfterReadycheck == nil then
			-- Set active settings to defaults
			BuffCheck_ActiveSettings.CheckAfterReadycheck = bc.Defaults.CheckAfterReadycheck
			bc.btnReadycheck:SetChecked(bc.Defaults.CheckAfterReadycheck)
		end
		-- Set state of checkbutton within interface options
		bc.btnReadycheck:SetChecked(BuffCheck_ActiveSettings.CheckAfterReadycheck)
		
		-- Display settings on load?
		if BuffCheck_ActiveSettings.DisplaySettingsOnLoad == nil then
			-- Set active settings to defaults
			BuffCheck_ActiveSettings.DisplaySettingsOnLoad = bc.Defaults.DisplaySettingsOnLoad
			bc.btnDisplaySettingsOnLoad:SetChecked(bc.Defaults.DisplaySettingsOnLoad)
		end
		-- Set state of checkbutton within interface options
		bc.btnDisplaySettingsOnLoad:SetChecked(BuffCheck_ActiveSettings.DisplaySettingsOnLoad)
		
		-- DISPLAY LOAD MESSAGE --
		bc.Display(bc.Description.Title.." v"..bc.Description.VersionNum.." loaded.")
		if BuffCheck_ActiveSettings.DisplaySettingsOnLoad then
			bc.ViewSettings()
		end
	
	end -- if event and addon
	
	if event == ("READY_CHECK_FINISHED") and (BuffCheck_ActiveSettings.CheckAfterReadycheck) then
		bc.Buffs.Clear()
		bc.CheckBuffs()
	end
	
end)

-- SCRIPTS FOR INTERFACE OPTIONS --============================================

-------------------------------------------------------------------------------
-- Changes the settings table to match the state of the 'disable output' checkbutton.
-------------------------------------------------------------------------------
bc.btnDisableOutput:SetScript("OnClick", function ()
	
	if bc.btnDisableOutput:GetChecked() then
		BuffCheck_ActiveSettings.DisableOutput = true
	else
		BuffCheck_ActiveSettings.DisableOutput = false
	end
	
end)

-------------------------------------------------------------------------------
-- Changes the settings table to match the state of the 'announce' checkbutton.
-------------------------------------------------------------------------------
bc.btnAnnounce:SetScript("OnClick", function ()
	
	if bc.btnAnnounce:GetChecked() then
		BuffCheck_ActiveSettings.AnnounceMissing = true
	else
		BuffCheck_ActiveSettings.AnnounceMissing = false
	end
	
end)

-------------------------------------------------------------------------------
-- Changes the settings table to match the state of the 'readycheck' checkbutton.
-------------------------------------------------------------------------------
bc.btnReadycheck:SetScript("OnClick", function ()
	
	if bc.btnReadycheck:GetChecked() then
		BuffCheck_ActiveSettings.CheckAfterReadycheck = true
	else
		BuffCheck_ActiveSettings.CheckAfterReadycheck = false
	end
	
end)

-------------------------------------------------------------------------------
-- Changes the settings table to match the state of the 'settings on load' checkbutton.
-------------------------------------------------------------------------------
bc.btnDisplaySettingsOnLoad:SetScript("OnClick", function ()
	
	if bc.btnDisplaySettingsOnLoad:GetChecked() then
		BuffCheck_ActiveSettings.DisplaySettingsOnLoad = true
	else
		BuffCheck_ActiveSettings.DisplaySettingsOnLoad = false
	end
	
end)

-------------------------------------------------------------------------------
-- Enables functionality of 'Defaults' button in the interface options.
-------------------------------------------------------------------------------
bc.panel.default = function()
	
	bc.SetDefaults()
	
end
